
#include <string.h>
#include <stdlib.h>

#include <gtk/gtk.h>

#include "world.h"
#include "add_stuff.h"
#include "edits.h"
#include "save_lists.h"
#include "icon.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


void cancel (GtkWidget *widget, GtkWidget *widget_to_close)
{
  gtk_widget_destroy (widget_to_close);
}


void accept_new_tab (GtkWidget *widget, world_info *info)
{
  const gchar *text;
  
  text = gtk_entry_get_text (GTK_ENTRY (info->new_tab_field));
  if ((text == NULL) || (strcmp (text, "") == 0))
    return;
  add_tab (info, text);
  gtk_widget_destroy (info->new_tab_dialog);
  gtk_notebook_set_current_page (GTK_NOTEBOOK (info->tabs), -1);
  save_all (info);
}


void new_tab (GtkWidget *widget, world_info *info)
{
  GtkWidget *label, *button;

  gtk_widget_destroy (info->options_dialog);
  info->new_tab_dialog = gtk_dialog_new_with_buttons ("New Tab",
    GTK_WINDOW (info->main_window), GTK_DIALOG_NO_SEPARATOR, NULL);
  gtk_window_set_transient_for (GTK_WINDOW (info->new_tab_dialog),
    GTK_WINDOW (info->main_window));
  gtk_window_set_modal (GTK_WINDOW (info->new_tab_dialog), TRUE);
  gtk_container_set_border_width (GTK_CONTAINER (info->new_tab_dialog),
    info->button_spacing);
  info->new_tab_table = gtk_table_new (3, 5, FALSE);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (info->new_tab_dialog)->vbox),
    info->new_tab_table);
  gtk_table_set_row_spacings (GTK_TABLE (info->new_tab_table),
    info->button_spacing);
  gtk_table_set_col_spacings (GTK_TABLE (info->new_tab_table),
    info->button_spacing);

  label = gtk_label_new ("New tab name:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (info->new_tab_table), label,
    0, 1, 0, 1);
  gtk_widget_show (label);

  info->new_tab_field = gtk_entry_new();
  gtk_table_attach_defaults (GTK_TABLE (info->new_tab_table),
    info->new_tab_field, 2, 3, 0, 1);
  gtk_widget_show (info->new_tab_field);
  
  button = gtk_button_new_with_label ("OK");
  gtk_widget_show (button);
  g_signal_connect (G_OBJECT (button), "clicked",
    G_CALLBACK (accept_new_tab), info);
  gtk_table_attach_defaults (GTK_TABLE (info->new_tab_table), button,
    4, 5, 0, 1);
  gtk_widget_show (button);
  
  button = gtk_button_new_with_label ("Cancel");
  gtk_widget_show (button);
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (cancel),
    info->new_tab_dialog);
  gtk_table_attach_defaults (GTK_TABLE (info->new_tab_table), button,
    4, 5, 2, 3);
  gtk_widget_show (button);
  
  gtk_widget_show_all (info->new_tab_dialog);
}


void accept_mod_tab (GtkWidget *widget, world_info *info)
{
  GtkWidget *table, *label;
  const gchar *text;
  gint tab_number;
  tab_struct *tab;

  tab_number = gtk_notebook_get_current_page (GTK_NOTEBOOK (info->tabs));
  text = gtk_entry_get_text (GTK_ENTRY (info->mod_tab_field));
  if ((text == NULL) || (strcmp (text, "") == 0))
    del_tab (info, tab_number);
  else
  {
    table = gtk_notebook_get_nth_page (GTK_NOTEBOOK (info->tabs), tab_number);
    label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (info->tabs), table);
    gtk_label_set_text (GTK_LABEL (label), text);
    tab = (tab_struct*) g_list_nth (info->tab_list, tab_number)->data;
    g_string_assign (tab->tab_name, text);
  }
  gtk_widget_destroy (info->mod_tab_dialog);
  save_all (info);
}


void mod_tab (GtkWidget *widget, world_info *info)
{
  GtkWidget *table, *label, *button;
  tab_struct *tab;
  gint tab_number;

  gtk_widget_destroy (info->options_dialog);
  info->mod_tab_dialog = gtk_dialog_new_with_buttons ("Modify Tab",
    GTK_WINDOW (info->main_window), GTK_DIALOG_NO_SEPARATOR, NULL);
  gtk_window_set_transient_for (GTK_WINDOW (info->mod_tab_dialog),
    GTK_WINDOW (info->main_window));
  gtk_window_set_modal (GTK_WINDOW (info->mod_tab_dialog), TRUE);
  gtk_container_set_border_width (GTK_CONTAINER (info->mod_tab_dialog),
    info->button_spacing);
  table = gtk_table_new (4, 5, FALSE);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (info->mod_tab_dialog)->vbox),
    table);
  gtk_table_set_row_spacings (GTK_TABLE (table), info->button_spacing);
  gtk_table_set_col_spacings (GTK_TABLE (table), info->button_spacing);

  label = gtk_label_new ("New name:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);

  info->mod_tab_field = gtk_entry_new();
  tab_number = gtk_notebook_get_current_page (GTK_NOTEBOOK (info->tabs));
  tab = (tab_struct*) g_list_nth (info->tab_list, tab_number)->data;
  gtk_entry_set_text (GTK_ENTRY (info->mod_tab_field), tab->tab_name->str);
  gtk_table_attach_defaults (GTK_TABLE (table), info->mod_tab_field,
    2, 3, 0, 1);
  
  button = gtk_button_new_with_label ("OK");
  g_signal_connect (G_OBJECT (button), "clicked",
    G_CALLBACK (accept_mod_tab), info);
  gtk_table_attach_defaults (GTK_TABLE (table), button, 4, 5, 0, 1);
  
  button = gtk_button_new_with_label ("Cancel");
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (cancel),
    info->mod_tab_dialog);
  gtk_table_attach_defaults (GTK_TABLE (table), button, 4, 5, 1, 2);
  
  label = gtk_label_new ("Remove name to delete tab.");
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 3, 2, 3);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  gtk_widget_show_all (info->mod_tab_dialog);
}


void delete_stuff (GtkWidget *widget, world_info *info)
{
  GString *message;
  
  gtk_widget_destroy (info->options_dialog);
  message = g_string_new (NULL);
  g_string_sprintf (message,
    "All delete features are not coded, but you may\n"
    "edit the text configuration file.\n"
    "Tabble is currently using this location:\n\n%s\n\n"
    "(Path may be relative to current working directory.)",
    info->conf_file->str);
  show_notice (message->str, info->main_window);
}


void dimensions_accept (GtkWidget *widget, world_info *info)
{
  gint x, y, t;
  
  x = atoi (gtk_entry_get_text (GTK_ENTRY (info->dim_x)));
  y = atoi (gtk_entry_get_text (GTK_ENTRY (info->dim_y)));

  if (x <=0 || y <= 0)
  {
    gtk_widget_destroy (info->dim_dialog);
    return;
  }

  info->x_icons_new = x;
  info->y_icons_new = y;
  save_all_compressed (info);

  for (t = info->tab_count - 1; t >= 0; t--)
    del_tab (info, t);
  info->run_again = TRUE;
  gtk_widget_destroy (info->dim_dialog);
  gtk_widget_destroy (info->main_window);
}


void dimensions (GtkWidget *widget, world_info *info)
{
  GtkWidget *table, *label, *button;
  GString *string;

  gtk_widget_destroy (info->options_dialog);
  info->dim_dialog = gtk_dialog_new_with_buttons ("Dimensions",
    GTK_WINDOW (info->main_window), GTK_DIALOG_NO_SEPARATOR, NULL);
  gtk_window_set_transient_for (GTK_WINDOW (info->dim_dialog),
    GTK_WINDOW (info->main_window));
  gtk_window_set_modal (GTK_WINDOW (info->dim_dialog), TRUE);
  gtk_container_set_border_width (GTK_CONTAINER (info->dim_dialog),
    info->button_spacing);
  table = gtk_table_new (4, 3, FALSE);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (info->dim_dialog)
    ->vbox), table);
  gtk_table_set_row_spacings (GTK_TABLE (table), info->button_spacing);
  gtk_table_set_col_spacings (GTK_TABLE (table), info->button_spacing);

  label = gtk_label_new ("X across:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
  
  label = gtk_label_new ("Y down:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);

  string = g_string_new (NULL);
    
  info->dim_x = gtk_entry_new();
  g_string_sprintf (string, "%d", info->x_icons);
  gtk_entry_set_text (GTK_ENTRY (info->dim_x), string->str);
  gtk_table_attach (GTK_TABLE (table), info->dim_x, 1, 2, 0, 1,
    GTK_SHRINK, GTK_SHRINK, 0, 0);

  info->dim_y = gtk_entry_new();
  g_string_sprintf (string, "%d", info->y_icons);
  gtk_entry_set_text (GTK_ENTRY (info->dim_y), string->str);
  gtk_table_attach (GTK_TABLE (table), info->dim_y, 1, 2, 1, 2,
    GTK_SHRINK, GTK_SHRINK, 0, 0);

  button = gtk_button_new_with_label ("OK");
  gtk_table_attach (GTK_TABLE (table), button, 2, 3, 0, 1,
    GTK_FILL, GTK_SHRINK, 0, 0);
  g_signal_connect (G_OBJECT (button), "clicked",
    G_CALLBACK (dimensions_accept), info);
  
  button = gtk_button_new_with_label ("Cancel");
  gtk_table_attach (GTK_TABLE (table), button, 2, 3, 1, 2,
    GTK_FILL, GTK_SHRINK, 0, 0);
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (cancel),
    info->dim_dialog);

  label = gtk_label_new ("NB: clicking OK will compress your\n"
    "programs linearily.");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 3, 2, 3);

  gtk_widget_show_all (info->dim_dialog);
}


void save_position (GtkWidget *widget, world_info *info)
{
  gtk_widget_destroy (info->options_dialog);
  gtk_window_get_position (GTK_WINDOW (info->main_window),
    &info->start_x, &info->start_y);
  save_all (info);
}


void accept_ontop_sticky (GtkWidget *widget, world_info *info)
{
  info->world_on_top =
    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (info->ontop_check));
  info->world_sticky =
    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (info->sticky_check));

  gtk_widget_destroy (info->ontop_sticky_dialog);
  
  gtk_window_set_keep_above (GTK_WINDOW (info->main_window),
    info->world_on_top);
  if (info->world_sticky)
    gtk_window_stick (GTK_WINDOW (info->main_window));
  else
    gtk_window_unstick (GTK_WINDOW (info->main_window));

  save_all (info);
}


void ontop_sticky (GtkWidget *widget, world_info *info)
{
  GtkWidget *button;

  gtk_widget_destroy (info->options_dialog);
  info->ontop_sticky_dialog = gtk_dialog_new_with_buttons ("Tabble Behaviour",
    GTK_WINDOW (info->main_window), GTK_DIALOG_NO_SEPARATOR, NULL);
  gtk_window_set_transient_for (GTK_WINDOW (info->ontop_sticky_dialog),
    GTK_WINDOW (info->main_window));
  gtk_window_set_modal (GTK_WINDOW (info->ontop_sticky_dialog), TRUE);
  gtk_container_set_border_width (GTK_CONTAINER (info->ontop_sticky_dialog),
    info->button_spacing);
  info->ontop_sticky_table = gtk_table_new (3, 3, FALSE);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (info->ontop_sticky_dialog)
    ->vbox), info->ontop_sticky_table);
  gtk_table_set_row_spacings (GTK_TABLE (info->ontop_sticky_table),
    info->button_spacing);
  gtk_table_set_col_spacings (GTK_TABLE (info->ontop_sticky_table),
    info->button_spacing);

  info->ontop_check = gtk_check_button_new_with_label ("Stay on top");
  gtk_table_attach_defaults (GTK_TABLE (info->ontop_sticky_table),
    info->ontop_check, 0, 1, 0, 1);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (info->ontop_check),
    info->world_on_top);
  gtk_widget_show (info->ontop_check);

  info->sticky_check = gtk_check_button_new_with_label ("All desktops");
  gtk_table_attach_defaults (GTK_TABLE (info->ontop_sticky_table),
    info->sticky_check, 0, 1, 2, 3);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (info->sticky_check),
    info->world_sticky);
  gtk_widget_show (info->sticky_check);
    
  button = gtk_button_new_with_label ("OK");
  g_signal_connect (G_OBJECT (button), "clicked",
    G_CALLBACK (accept_ontop_sticky), info);
  gtk_table_attach_defaults (GTK_TABLE (info->ontop_sticky_table), button,
    2, 3, 0, 1);
  gtk_widget_show (button);
  
  button = gtk_button_new_with_label ("Cancel");
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (cancel),
    info->ontop_sticky_dialog);
  gtk_table_attach_defaults (GTK_TABLE (info->ontop_sticky_table), button,
    2, 3, 2, 3);
  gtk_widget_show (button);
  
  gtk_widget_show_all (info->ontop_sticky_dialog);
}


void add (world_info *info, gint row, gchar *button_text, gchar *label_text,
          GCallback function)
{
  GtkWidget *button, *label;
  
  button = gtk_button_new_with_label (button_text);
  gtk_table_attach_defaults (GTK_TABLE (info->options_table), button,
    0, 1, row, row + 1);
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (function), info);
  gtk_widget_show (button);
  
  label = gtk_label_new (label_text);
  gtk_table_attach_defaults (GTK_TABLE (info->options_table), label,
    2, 3, row, row + 1);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_widget_show (label);
}


void about (GtkWidget *widget, world_info *info)
{
  GtkWidget *dialog, *table, *icon, *label, *field, *button;
  GdkPixbuf *pix_buf;
  GString *string;

  gtk_widget_destroy (info->options_dialog);
  
  dialog = gtk_dialog_new_with_buttons ("About",
    GTK_WINDOW (info->main_window), GTK_DIALOG_NO_SEPARATOR, NULL);
  gtk_window_set_transient_for (GTK_WINDOW (dialog),
    GTK_WINDOW (info->main_window));
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
  gtk_container_set_border_width (GTK_CONTAINER (dialog),
    info->button_spacing);
  table = gtk_table_new (6, 5, FALSE);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), table);
  gtk_table_set_row_spacings (GTK_TABLE (table), info->button_spacing);
  gtk_table_set_col_spacings (GTK_TABLE (table), info->button_spacing);

  pix_buf = gdk_pixbuf_new_from_xpm_data (tabble_icon());
  if (pix_buf)
  {
    icon = gtk_image_new_from_pixbuf (pix_buf);
    if (icon)
    {
      gtk_table_attach (GTK_TABLE (table), icon,
       0, 1, 2, 3, GTK_SHRINK, GTK_SHRINK, info->frame_spacing, 0);
      gtk_widget_show (icon);
    }
  }

  string = g_string_new (NULL);
  g_string_sprintf (string, "%s %s\n\n"
    "Distributed under the GPL this program is\n"
    "copyright (c) 2005, 2006 Graeme Sheppard.\n\n"
    "Homepage:\n", PACKAGE, VERSION);
  
  label = gtk_label_new (string->str);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 2, 3, 2, 3);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_widget_show (label);

  field = gtk_entry_new();
  gtk_entry_set_text (GTK_ENTRY (field), "http://www.rillion.net/tabble/");
  gtk_entry_set_editable (GTK_ENTRY (field), FALSE);
  gtk_table_attach_defaults (GTK_TABLE (table), field, 2, 3, 3, 4);
  gtk_widget_show (field);

  button = gtk_button_new_with_label ("OK");
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (dismiss_notice),
    dialog);
  gtk_table_attach (GTK_TABLE (table), button, 4, 5, 5, 6,
    GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_widget_show (button);

  gtk_widget_show_all (dialog);
}


void options_dialog (world_info *info)
{
  GtkWidget *button, *label;

  info->options_dialog = gtk_dialog_new_with_buttons ("Options",
    GTK_WINDOW (info->main_window), GTK_DIALOG_NO_SEPARATOR, NULL);
  gtk_window_set_transient_for (GTK_WINDOW (info->options_dialog),
    GTK_WINDOW (info->main_window));
  gtk_window_set_modal (GTK_WINDOW (info->options_dialog), TRUE);
  gtk_container_set_border_width (GTK_CONTAINER (info->options_dialog),
    info->button_spacing);
  info->options_table = gtk_table_new (10, 4, FALSE);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (info->options_dialog)->vbox),
    info->options_table);
  gtk_table_set_row_spacings (GTK_TABLE (info->options_table),
    info->frame_spacing);
  gtk_table_set_col_spacings (GTK_TABLE (info->options_table),
    info->frame_spacing * 2);

  add (info, 0, "New Tab", "Create a new, empty tab",
    G_CALLBACK (new_tab));
  add (info, 1, "Modify tab", "Rename or delete current tab",
    G_CALLBACK (mod_tab));
  add (info, 2, "Delete", "How to remove other objects",
    G_CALLBACK (delete_stuff));
  add (info, 3, "Dimensions", "Set rows and columns",
    G_CALLBACK (dimensions));
  add (info, 4, "Position", "Save current screen position",
    G_CALLBACK (save_position));
  add (info, 5, "Behaviour", "On top and sticky options",
    G_CALLBACK (ontop_sticky));
  add (info, 6, "About", "",
    G_CALLBACK (about));

  label = gtk_label_new
  (
    "\n"
    "To make empty cells visible press shift.\n"
    "To edit a program right click the button."
  );
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 1.0);
  gtk_table_attach_defaults (GTK_TABLE (info->options_table), label,
    0, 3, 7, 10);
  gtk_widget_show (label);
  
  button = gtk_button_new_with_label ("Cancel");
  gtk_widget_show (button);
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (cancel),
    info->options_dialog);
  gtk_table_attach_defaults (GTK_TABLE (info->options_table), button,
    3, 4, 9, 10);
  gtk_widget_show (button);
  
  gtk_widget_show_all (info->options_dialog);
}

